#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/time.h>

#define N 10000000

#define ERROR(msg)				\
  do {						\
    perror(msg);				\
    exit(EXIT_FAILURE);				\
  } while (0)

typedef unsigned short int value;

static double wtime (void) {
  struct timeval now;

  (void) gettimeofday(&now, NULL);

  return now.tv_sec + 1e-6*now.tv_usec;
}

static void loop_copy (const value * src, size_t n) {
  size_t i;
  value * dst;
  double timer;

  dst = malloc(n * sizeof(value));

  if (dst == NULL)
    ERROR("malloc");

  timer = wtime();
  for (i = 0; i < n; i++)
    dst[i] = src[i];
  timer = wtime() - timer;

  printf("loop copy: %.5f\n", timer);

  free(dst);
}

static void memcpy_copy (const value * src, size_t n) {
  value * dst;
  double timer;

  dst = malloc(n * sizeof(value));

  if (dst == NULL)
    ERROR("malloc");

  timer = wtime();
  memcpy(dst, src, n * sizeof(value));
  timer = wtime() - timer;

  printf("memcpy copy: %.5f\n", timer);

  free(dst);
}

int main (void) {
  size_t i;
  value * values;

  values = malloc(N * sizeof(value));

  if (values == NULL)
    ERROR("malloc");

  for (i = 0; i < N; i++)
    values[i] = i;

  loop_copy(values, N);
  memcpy_copy(values, N);

  free(values);
  return 0;
}
